use crate::config::Context;
use crate::zabbix::api::get_zabbix_problems;
use launchy::Color;
use launchy::{self, Canvas, CanvasLayout, CanvasLayoutPoller, MsgPollingWrapper, Pad};
use std::thread::sleep;
use std::time::Duration;

pub const MATRIX_WIDTH: i32 = 8;
pub const MATRIX_SIZE: i32 = MATRIX_WIDTH * MATRIX_WIDTH;
pub const REQUEST_TIMEOUT: i32 = 5;
pub const ZERO_COLOR: f32 = 0.;
pub const ONE_COLOR: f32 = 1.;
pub const SLOWEFFECT_FACTOR: f32 = 800.0;

pub fn run(ctx: &mut Context) {
    match ctx.mode.as_str() {
        "draw" => {
            let (mut canvas, mut _poller) = initpad();
            draw(&mut canvas, ctx);
        }
        "input" => {
            let (mut canvas, mut poller) = initpad();
            input(&mut canvas, &mut poller);
        }
        "effect" => {
            let (mut canvas, mut poller) = initpad();
            effect(&mut canvas, &mut poller)
        }
        "test" => test(ctx),
        _ => {
            println!("No valid option choosen for mode");
            std::process::exit(1)
        }
    }
}

fn initpad() -> (CanvasLayout<'static>, CanvasLayoutPoller) {
    let (mut canvas, poller) = launchy::CanvasLayout::new_polling();
    match canvas.add_by_guess_rotated::<launchy::mini::Canvas>(0, 0, launchy::Rotation::None) {
        Ok(o) => {
            println!("Connected to device using USB {o:?}");
            clear(&mut canvas);
        }
        Err(err) => {
            eprintln!("Failed to connect to device, check USB connection {err}",);
        }
    };
    (canvas, poller)
}

fn input(canvas: &mut CanvasLayout, poller: &mut CanvasLayoutPoller) {
    for color in (0u64..).map(|f| Color::red_green_color(f as f32 / 60.0 / 2.5)) {
        for msg in poller.iter_for_millis(17).filter(|msg| msg.is_press()) {
            canvas[msg.pad()] = color;
            println!("{msg:?}", msg = msg.pad())
        }
        canvas.flush().unwrap();
    }
}

fn test(ctx: &mut Context) {
    println!(
        "Refresh rate is {sec} seconds",
        sec = ctx.cfg.refresh.unwrap()
    );
    loop {
        ctx.hotreload();
        let zabbix_data = match get_zabbix_problems(&ctx.cfg) {
            Ok(z) => z,
            Err(_) => {
                println!(
                    "Error requesting zabbix service, retrying in {} seconds",
                    REQUEST_TIMEOUT
                );
                sleep(Duration::from_secs(REQUEST_TIMEOUT as u64));
                continue;
            }
        };
        println!("{}", zabbix_data);
    }
}

pub fn draw(canvas: &mut CanvasLayout, ctx: &mut Context) {
    println!("Refresh rate is {} seconds", ctx.cfg.refresh.unwrap());
    loop {
        let mut max = 0;
        let zabbix_data = match get_zabbix_problems(&ctx.cfg) {
            Ok(zabbix_data) => zabbix_data,
            Err(_) => {
                println!(
                    "Error requesting zabbix service, retrying in {}",
                    REQUEST_TIMEOUT
                );
                sleep(Duration::from_secs(REQUEST_TIMEOUT as u64));
                continue;
            }
        };
        ctx.datamatrix.compute(&zabbix_data);
        clear(canvas);

        for (i, j) in ctx.datamatrix.layout.iter().enumerate() {
            let p = Pad {
                x: ((i as i32) % MATRIX_WIDTH),
                y: ((i as i32) / MATRIX_WIDTH) + 1,
            };
            let c = match j.status {
                5 => Color::RED,
                4 => Color::from_hue(0.05),
                3 => Color::from_hue(0.1),
                2 => Color::from_hue(1.22),
                _ => Color::GREEN,
            };
            canvas[p] = c;
            if ctx.cfg.sloweffect.unwrap() {
                sleep(Duration::from_millis(
                    (SLOWEFFECT_FACTOR * (1.0 / ctx.datamatrix.layout.len() as f32)) as u64,
                ));
            }
            canvas.flush().unwrap();
            max += 1;
            if max >= MATRIX_SIZE {
                break;
            }
        }
        ctx.hotreload();
    }
}

fn effect(canvas: &mut CanvasLayout, poller: &mut CanvasLayoutPoller) {
    for color in (0u64..).map(|f| Color::red_green_color(f as f32 / 60.0 / 2.5)) {
        for msg in poller.iter_for_millis(17).filter(|msg| msg.is_press()) {
            canvas[msg.pad()] = color;
            println!("{msg:?}", msg = msg.pad())
        }
        canvas.flush().unwrap();

        canvas.iter().for_each(|pad| {
            let surrounding_color = pad
                .neighbors_5()
                .iter()
                .map(|&p| canvas.get(p).unwrap_or(Color::GREEN))
                .sum::<Color>()
                / 5.0
                / 1.05;

            canvas[pad] = canvas[pad].mix(surrounding_color, 0.4);
        });
    }
}

fn clear(canvas: &mut CanvasLayout) {
    for a in 0..MATRIX_WIDTH {
        for b in 0..MATRIX_WIDTH + 1 {
            canvas[Pad { x: a, y: b }] = Color {
                r: ZERO_COLOR,
                g: ZERO_COLOR,
                b: ONE_COLOR,
            };
        }
    }
    canvas.flush().unwrap();
}
